/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): Vivien Quema
 */

package helloworld;

import org.objectweb.dream.Push;
import org.objectweb.dream.control.activity.task.TaskManagerController;
import org.objectweb.dream.message.manager.MessageManagerImpl;
import org.objectweb.dream.message.manager.MessageManager;
import org.objectweb.dream.message.manager.MessageManagerAttributeController;
import org.objectweb.fractal.api.Component;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.ContentController;
import org.objectweb.fractal.api.factory.GenericFactory;
import org.objectweb.fractal.api.type.ComponentType;
import org.objectweb.fractal.api.type.InterfaceType;
import org.objectweb.fractal.api.type.TypeFactory;
import org.objectweb.fractal.util.Fractal;

/** Launcher class. */
public final class Main
{

  private Main()
  {
  }

  /**
   * Entry point
   * 
   * @param args if no argument or one equals to <code>-mono</code> use the
   *          {@link ClientImpl}. If equals to <code>-pool</code> the the
   *          {@link PoolClientImpl}.
   * @throws Exception if an exception occurs
   */
  public static void main(String[] args) throws Exception
  {
    if (args.length > 1)
    {
      usage();
    }
    String clientClass = null;
    if (args.length == 0)
    {
      clientClass = ClientImpl.class.getName();
    }
    else
    {
      String arg = args[0];
      if (arg.equalsIgnoreCase("-mono"))
      {
        clientClass = ClientImpl.class.getName();
      }
      else if (arg.equalsIgnoreCase("-pool"))
      {
        clientClass = PoolClientImpl.class.getName();
      }
      else
      {
        usage();
      }
    }

    Component bootstrap = org.objectweb.fractal.api.Fractal
        .getBootstrapComponent();
    GenericFactory genericFactory = Fractal.getGenericFactory(bootstrap);
    TypeFactory typeFactory = Fractal.getTypeFactory(bootstrap);

    // create interface types
    InterfaceType outPushType = typeFactory.createFcItfType(
        Push.OUT_PUSH_ITF_NAME, Push.class.getName(), true, false, false);
    InterfaceType inPushType = typeFactory.createFcItfType(
        Push.IN_PUSH_ITF_NAME, Push.class.getName(), false, false, false);
    InterfaceType messageManagerClientType = typeFactory.createFcItfType(
        MessageManager.ITF_NAME, MessageManager.class.getName(), true, false,
        false);
    InterfaceType messageManagerServerType = typeFactory.createFcItfType(
        MessageManager.ITF_NAME, MessageManager.class.getName(), false, false,
        false);
    InterfaceType messageManagerAttributeControllerType = typeFactory
        .createFcItfType("attribute-controller",
            MessageManagerAttributeController.class.getName(), false, false,
            false);
    InterfaceType taskRegistrationClientType = typeFactory.createFcItfType(
        "task-manager", TaskManagerController.class.getName(), true, false,
        false);

    // create component types
    ComponentType clientType = typeFactory.createFcType(new InterfaceType[]{
        outPushType, messageManagerClientType, taskRegistrationClientType});
    ComponentType serverType = typeFactory.createFcType(new InterfaceType[]{
        inPushType, messageManagerClientType});
    ComponentType messageManagerType = typeFactory
        .createFcType(new InterfaceType[]{messageManagerServerType,
            messageManagerAttributeControllerType});
    ComponentType compositeType = typeFactory
        .createFcType(new InterfaceType[0]);

    // instanciate components
    Component client = genericFactory.newFcInstance(clientType,
        "activeDreamPrimitive", clientClass);
    Component server = genericFactory.newFcInstance(serverType,
        "dreamPrimitive", ServerImpl.class.getName());
    Component messageManager = genericFactory.newFcInstance(messageManagerType,
        "dreamPrimitive", MessageManagerImpl.class.getName());
    Component composite = genericFactory.newFcInstance(compositeType,
        "dreamComposite", null);
    Component activityManager = genericFactory.newFcInstance(compositeType,
        "threadPerTaskActivityComposite", null);

    // set component names
    Fractal.getNameController(composite).setFcName("HelloWorld");
    Fractal.getNameController(client).setFcName("client");
    Fractal.getNameController(server).setFcName("server");
    Fractal.getNameController(messageManager).setFcName("messageManager");
    Fractal.getNameController(activityManager).setFcName("activityManager");

    // set component attributes
    MessageManagerAttributeController mmac = (MessageManagerAttributeController) messageManager
        .getFcInterface("attribute-controller");
    mmac.setId((short) 0);

    // add sub components in composite.
    ContentController compositeCC = Fractal.getContentController(composite);
    compositeCC.addFcSubComponent(client);
    compositeCC.addFcSubComponent(server);
    compositeCC.addFcSubComponent(messageManager);
    compositeCC.addFcSubComponent(activityManager);

    // bind them
    BindingController clientBC = Fractal.getBindingController(client);
    clientBC.bindFc(Push.OUT_PUSH_ITF_NAME, server
        .getFcInterface(Push.IN_PUSH_ITF_NAME));
    clientBC.bindFc(MessageManager.ITF_NAME, messageManager
        .getFcInterface(MessageManager.ITF_NAME));
    clientBC.bindFc("task-manager", activityManager
        .getFcInterface("task-manager-controller"));
    BindingController serverBC = Fractal.getBindingController(server);
    serverBC.bindFc(MessageManager.ITF_NAME, messageManager
        .getFcInterface(MessageManager.ITF_NAME));

    // and start
    Fractal.getLifeCycleController(composite).startFc();
    Thread.sleep(3000);
    System.out.println("call to stop...");
    Fractal.getLifeCycleController(composite).stopFc();
    System.out.println("stop returned");
  }

  private static void usage()
  {
    System.out.println("Usage : HelloWorld [-mono|-pool]");
    System.out.println("Run the Helloworld example of Dream core framework");
    System.out.println("Arguments : ");
    System.out
        .println("  -mono : use the mono-threaded client component (default) ");
    System.out.println("  -pool : use the client component using thread pool");
    System.exit(1);
  }
}