/**
 * Dream Copyright (C) 2003-2004 INRIA Rhone-Alpes
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option) any
 * later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * Contact: dream@objectweb.org
 * 
 * Initial developer(s): Matthieu Leclercq 
 * Contributor(s):
 */

package org.objectweb.dream.control.activity.task;

import java.util.Map;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.util.monolog.api.BasicLevel;
import org.objectweb.util.monolog.api.Logger;

/**
 * Basic Implementation of the {@link TaskActivationController}. When a task is
 * activated, it registers it in the {@link TaskManagerController }this
 * component is bound to. This mixin must be used with a {@link TaskController }
 * implementation mixin like {@link BasicTaskControllerMixin}.<br>
 * <br>
 * <b>Requirements </b>
 * <ul>
 * <li>this mixin requires the {@link LoggableTaskMixin }mixin.</li>
 * <li>this mixin requires the
 * {@link org.objectweb.fractal.julia.control.binding.UseBindingControllerMixin}
 * mixin.</li>
 * </ul>
 */
public abstract class BasicTaskActivationMixin
    implements
      TaskActivationController
{

  // ---------------------------------------------------------------------------
  // Implementation of the TaskActivationController interface
  // ---------------------------------------------------------------------------

  /**
   * @see TaskActivationController#activateTask(Task)
   */
  public void activateTask(Task task) throws NoSuchTaskException,
      IllegalTaskException
  {
    if (!_this_taskRegistrationHints.containsKey(task))
    {
      throw new NoSuchTaskException(task);
    }
    if (_this_taskControls.containsKey(task))
    {
      return;
    }
    _this_weaveableTCLogger.log(BasicLevel.DEBUG, "register a task");
    Map hints = (Map) _this_taskRegistrationHints.get(task);
    TaskManagerController tmc = getTaskManagerBinding();
    Object taskControl;
    try
    {
      taskControl = tmc.registerTask(task, hints);
    }
    catch (Exception e)
    {
      throw new IllegalTaskException(task, "Unable to register component", e);
    }

    _this_taskControls.put(task, taskControl);
    _this_weaveableTCLogger.log(BasicLevel.DEBUG, "task registered");
  }

  /**
   * @see TaskActivationController#deactivateTask(Task)
   */
  public void deactivateTask(Task task) throws NoSuchTaskException,
      IllegalTaskException
  {
    if (!_this_taskRegistrationHints.containsKey(task))
    {
      throw new NoSuchTaskException(task);
    }
    if (!_this_taskControls.containsKey(task))
    {
      return;
    }
    _this_weaveableTCLogger.log(BasicLevel.DEBUG, "deactivate a task");
    try
    {
      getTaskManagerBinding().unregisterTask(task);
    }
    catch (Exception e)
    {
      throw new IllegalTaskException(task, "Unable to unregister component", e);
    }
    _this_taskControls.remove(task);
    _this_weaveableTCLogger.log(BasicLevel.DEBUG, "task deactivated");
  }

  /**
   * @see TaskActivationController#deactivateTask(Task, TaskStoppedListener)
   */
  public void deactivateTask(Task task, TaskStoppedListener taskStoppedListener)
      throws NoSuchTaskException, IllegalTaskException
  {
    if (!_this_taskRegistrationHints.containsKey(task))
    {
      throw new NoSuchTaskException(task);
    }
    if (!_this_taskControls.containsKey(task))
    {
      return;
    }
    _this_weaveableTCLogger.log(BasicLevel.DEBUG, "deactivate a task");
    try
    {
      getTaskManagerBinding().interruptTask(
          task,
          new TaskActivationStoppedListener(taskStoppedListener,
              _this_taskControls));
    }
    catch (Exception e)
    {
      throw new IllegalTaskException(task, "Unable to unregister component", e);
    }
    _this_taskControls.remove(task);
    _this_weaveableTCLogger.log(BasicLevel.DEBUG, "task deactivated");
  }

  // ---------------------------------------------------------------------------
  // Utility methods
  // ---------------------------------------------------------------------------

  private TaskManagerController getTaskManagerBinding()
      throws IllegalTaskException
  {
    try
    {
      return (TaskManagerController) _this_weaveableBC.lookupFc("task-manager");
    }
    catch (NoSuchInterfaceException e)
    {
      throw new IllegalTaskException(null,
          "This component has no task-manager client interface", null);
    }
  }

  // ---------------------------------------------------------------------------
  // Fields and methods required by the mixin class in the base class
  // ---------------------------------------------------------------------------

  /**
   * The <code>weaveableTCLogger</code> field required by this mixin. This
   * field is supposed to reference the logger of this controller.
   */
  public Logger            _this_weaveableTCLogger;

  /**
   * The <code>taskRegistrationHints</code> field required by this mixin. This
   * field is supposed to reference the map containing hints passed when task
   * are added in the task controller
   * 
   * @see TaskController#addTask(Task, Map)
   */
  public Map               _this_taskRegistrationHints;

  /**
   * The <code>taskControls</code> field required by this mixin. This field is
   * supposed to reference the map containing control object returned by
   * {@link TaskController#getTaskControl(Task)}.
   */
  public Map               _this_taskControls;

  /**
   * The <tt>weaveableBC</tt> field required by this mixin. This field is
   * supposed to reference the
   * {@link org.objectweb.fractal.api.control.BindingController}interface of
   * the component to which this controller object belongs.
   */
  public BindingController _this_weaveableBC;

}