/**
 * Dream
 * Copyright (C) 2003-2004 INRIA Rhone-Alpes
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: dream@objectweb.org
 *
 * Initial developer(s): Matthieu Leclercq
 * Contributor(s): 
 */

package org.objectweb.dream.message.codec;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.Externalizable;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.util.Iterator;

import org.objectweb.dream.message.Chunk;
import org.objectweb.dream.message.ChunkAlreadyExistsException;
import org.objectweb.dream.message.ChunkType;
import org.objectweb.dream.message.ExtensibleMessage;
import org.objectweb.dream.message.Message;
import org.objectweb.dream.message.MessageNC;
import org.objectweb.dream.message.MessageType;
import org.objectweb.dream.message.MessageTypeImpl;
import org.objectweb.dream.message.MessageTypeNCImpl;
import org.objectweb.dream.message.manager.MessageManager;
import org.objectweb.dream.pool.ObjectPool;
import org.objectweb.dream.pool.Recyclable;
import org.objectweb.dream.util.Error;
import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;

/**
 * Optimized implementation of message codec for extensible message. <br>
 * Requirements :
 * <ul>
 * <li>Decoded messages (ie. newly created message returned by the message
 * manager) must implement {@link ExtensibleMessage}</li>
 * <li>Messages must provide correct message types (ie. the
 * {@link Message#getMessageType() }method must be implemented)</li>
 * <li>Every chunks must implements interface {@link Externalizable}</li>
 * <li>Decoded chunks (ie. newly created chunks returned by the message
 * manager) must implements {@link Chunk }interface</li>
 * <li>Chunk types must implements {@link Externalizable}</li>
 * </ul>
 */
public class MessageCodecOptimizedStreamImpl
    extends
      AbstractMessageCodecObjectStream
{
  // ---------------------------------------------------------------------------
  // Client interface
  // ---------------------------------------------------------------------------

  /** The name of the {@link ChunkType }pool client interface name */
  public static final String CHUNK_TYPE_POOL_ITF_NAME = "chunk-type-pool";
  protected ObjectPool       chunkTypePoolItf         = null;

  // ---------------------------------------------------------------------------
  // Implementation of the AbstractMessageCodecDataStream abstract methods
  // ---------------------------------------------------------------------------

  /**
   * @see AbstractMessageCodecDataStream#doDecodeSingleMessage(DataInput)
   */
  protected Message doDecodeSingleMessage(DataInput input) throws IOException
  {
    ObjectInput objectInput = (ObjectInput) input;
    ExtensibleMessage message;
    boolean messageNC = objectInput.readBoolean();
    if (messageNC)
    {
      message = (ExtensibleMessage) messageManagerItf
          .createMessage(MessageTypeNCImpl.EMPTY_MESSAGE_TYPE_NC);
    }
    else
    {
      message = (ExtensibleMessage) messageManagerItf
          .createMessage(MessageTypeImpl.EMPTY_MESSAGE_TYPE);
    }

    ChunkType chunkType = (ChunkType) chunkTypePoolItf.newInstance();
    boolean hasMoreChunk = objectInput.readBoolean();
    try
    {
      while (hasMoreChunk)
      {
        String chunkName = objectInput.readUTF();
        ((Externalizable) chunkType).readExternal(objectInput);
        Externalizable chunk = (Externalizable) messageManagerItf
            .createChunk(chunkType);
        chunk.readExternal(objectInput);
        message.addChunk(chunkName, ((Chunk) chunk).getType(), chunk);
        hasMoreChunk = objectInput.readBoolean();
      }
    }
    catch (ClassNotFoundException e)
    {
      throw new IOException("Unable to decode message : " + e.getMessage());
    }
    catch (ChunkAlreadyExistsException e)
    {
      // cannot happen
      Error.bug(logger, e);
    }
    finally
    {
      chunkTypePoolItf.recycleInstance((Recyclable) chunkType);
    }
    message.setMessageManagerId(messageManagerItf.getMessageManagerId());
    return message;
  }

  /**
   * @see AbstractMessageCodecDataStream#doEncodeSingleMessage(Message,
   *      DataOutput)
   */
  protected void doEncodeSingleMessage(Message message, DataOutput output)
      throws IOException
  {
    ObjectOutput objectOutput = (ObjectOutput) output;
    if (message instanceof MessageNC)
    {
      objectOutput.writeBoolean(true);
    }
    else
    {
      objectOutput.writeBoolean(false);
    }
    MessageType type = message.getMessageType();
    Iterator iterator = type.getChunkNamesIterator();
    while (iterator.hasNext())
    {
      String chunkName = (String) iterator.next();
      ChunkType chunkType = type.getChunkType(chunkName);
      Object chunk = message.getChunk(chunkName);
      objectOutput.writeBoolean(true);
      objectOutput.writeUTF(chunkName);
      ((Externalizable) chunkType).writeExternal(objectOutput);
      ((Externalizable) chunk).writeExternal(objectOutput);
    }
    objectOutput.writeBoolean(false);
  }

  // ---------------------------------------------------------------------------
  // Implementation of BindingController interface
  // ---------------------------------------------------------------------------

  /**
   * @see org.objectweb.fractal.api.control.BindingController#listFc()
   */
  public String[] listFc()
  {
    return new String[]{MessageManager.ITF_NAME, CHUNK_TYPE_POOL_ITF_NAME};
  }

  /**
   * @see org.objectweb.fractal.api.control.BindingController#bindFc(String,
   *      Object)
   */
  public synchronized void bindFc(String clientItfName, Object serverItf)
      throws NoSuchInterfaceException, IllegalBindingException,
      IllegalLifeCycleException
  {
    super.bindFc(clientItfName, serverItf);
    if (clientItfName.equals(CHUNK_TYPE_POOL_ITF_NAME))
    {
      chunkTypePoolItf = (ObjectPool) serverItf;
    }
  }

}